import $ from 'jquery'
import { Plugin } from 'foundation-sites/js/foundation.core.plugin'
import { debounce } from '@moreonion/js-utils/src/utils'
import throttle from 'lodash.throttle'

/**
 * AdaptiveSticky module.
 *
 * @module foundation.adaptiveSticky
 */

class StickyButton extends Plugin {
  /**
   * Creates a new instance.
   * @class
   * @name StickyButton
   * @fires StickyButton#init
   * @param {Object} element - jQuery object to add the trigger to.
   * @param {Object} options - Overrides to the default plugin settings.
   */
  _setup(element, options = {}) {
    this.$element = element
    this.className = 'StickyButton'
    // defaults, fallbacks
    this.options = $.extend(true, {}, StickyButton.defaults, this.$element.data(), options)
    this.$blockingElement = $(this.options.blocker)

    this._log('setup', this)
    this._init()
  }

  /**
   * Initializes the plugin and calls functions to get functioning on load.
   * @function
   * @private
   */
  _init() {
    // Show (or hide) sticky buttons.
    if (this.$blockingElement.length) {
      $(window).on('scroll.zf.stickyButton resize.zf.stickyButton', throttle(() => {
        this._isNotOver(this.$blockingElement) ? this._show() : this._hide()
      }, 100))
      this._isNotOver(this.$blockingElement) ? this._show() : this._hide()
    } else {
      $(window).on('resize.zf.stickyButton', throttle(() => this._show(), 100))
      this._show()
    }
    this.$element.find('a').on('click', () => this._hide())
    this._log('init', this)
  }

  /**
   * Check if we’re not (vertically) scrolling over an element.
   * @function
   * @param {jQuery} $element The element to check.
   * @return {boolean} Whether scrollTop is not overlapping the element.
   * @private
   */
  _isNotOver ($element) {
    return this._isAfter($element) || this._isBefore($element)
  }

  /**
   * Check if we’ve (vertically) not yet reached an element.
   * @function
   * @param {jQuery} $element The element to check.
   * @return {boolean} Whether scrollTop is above the element.
   * @private
   */
  _isBefore ($element) {
    const elementTop = $element.offset().top
    this._log('still before', $element)
    return elementTop > $(window).scrollTop() + $(window).height() * (1 - this.options.topOverlap)
  }

  /**
   * Check if we’ve (vertically) scrolled down past an element.
   * @function
   * @param {jQuery} $element The element to check.
   * @return {boolean} Whether scrollTop is below the element.
   * @private
   */
  _isAfter ($element) {
    const elementBottom = $element.offset().top + $element.outerHeight()
    this._log('scrolled passed', $element)
    return elementBottom < $(window).scrollTop() + $(window).height() * this.options.bottomOverlap
  }

  /**
   * Reveal the sticky button.
   * @function
   * @fires stickyButton#show
   * @fires stickyButton#shown
   * @private
   */
  _show () {
    this._log('show', this.$element)
    /**
     * Fires when the showing starts.
     * @event stickyButton#show
     */
    this.$element.trigger('show.zf.stickyButton')
    this.$element.filter('.top').show(0)
      // Prevent the element from taking up space in the DOM.
      .css('margin-bottom', -1 * this.$element.filter('.top').outerHeight())
      .css('top', 0).css('opacity', 1)
    this.$element.filter('.bottom').show(0).css('bottom', 0).css('opacity', 1)
    /**
     * Fires when showing is finished.
     * @event stickyButton#shown
     */
    this.$element.trigger('shown.zf.stickyButton')
  }

  /**
   * Hide the sticky button.
   * @function
   * @fires stickyButton#hide
   * @fires stickyButton#hidden
   * @private
   */
  _hide () {
    this._log('hide', this.$element)
    /**
     * Fires when the hiding starts.
     * @event stickyButton#hide
     */
    this.$element.trigger('hide.zf.stickyButton')
    this.$element.filter('.top').css('top', -1 * this.$element.filter('.top').outerHeight()).css('opacity', 0)
    this.$element.filter('.bottom').css('bottom', -1 * this.$element.filter('.bottom').outerHeight()).css('opacity', 0)
    // Wait for CSS transitions to finish before hiding the element.
    // Need to use a specific transition property, the compound property is always empty
    if (this.$element.css('transition-duration')) {
      this.$element.on('transitionend.zf.stickyButton', () => {
        // Need to guard against edge case where this transition listener would
        // trigger on a show transition
        if (!this._isNotOver(this.$blockingElement)) {
          this.$element.hide(0)
        }
        this.$element.off('transitionend.zf.stickyButton')
      })
    } else {
      this.$element.hide(0)
    }

    /**
     * Fires when the hiding is finished.
     * @event stickyButton#hidden
     */
    this.$element.trigger('hidden.zf.stickyButton')
  }

  /**
   * Logs debug output to the console.
   * @function
   * @private
   */
  _log () {
    if (this.options.debug) {
      console.debug('[StickyButton]', ...arguments)
    }
  }

  /**
   * Destroys an instance.
   * @function
   */
  _destroy() {
    $(window).off('.zf.stickyButton')
    this._hide()
    this._log('destroyed', this)
  }
}

/**
 * Default settings for plugin.
 */
StickyButton.defaults = {
  /**
   * The selector of an element before and after which the sticky button is shown.
   * @option
   * @type {string|null}
   * @default 'null'
   */
  blocker: null,

  /**
   * The blocker element is allowed to reach into the viewport up to this percentage of the
   * viewport height before the button is hidden. 0 means no intersection, 1 the element covers
   * the entire viewport.
   * @option
   * @type {number}
   * @default '0'
   */
  topOverlap: 0,

  /**
   * The blocker element is allowed to still reach into the viewport up to this percentage of the
   * viewport height when the button is shown. 0 means no intersection, 1 the element covers
   * the entire viewport.
   * @option
   * @type {number}
   * @default '0'
   */
  bottomOverlap: 0,

  /**
   * Toggle debug output.
   * @option
   * @type {boolean}
   * @default false
   */
  debug: false
}

export { StickyButton }
